@@ -1,15 +1,18 @@
-import re
 import functools
+from datetime import datetime
 from django.conf import settings
-from django.core.paginator import Paginator, EmptyPage, PageNotAnInteger
-from restlib2.params import Parametizer, param_cleaners
+from django.core.cache import cache
+from restlib2.params import Parametizer
 from restlib2.resources import Resource
 from avocado.models import DataContext, DataView, DataQuery
 from ..decorators import check_auth
 from .. import cors
 
+__all__ = ('BaseResource', 'ThrottledResource')
+
 SAFE_METHODS = ('GET', 'HEAD', 'OPTIONS')
 
+
 def _get_request_object(request, attrs=None, klass=None, key=None):
     """Resolves the appropriate object for use from the request.
 
@@ -37,14 +40,14 @@ def _get_request_object(request, attrs=None, klass=None, key=None):
         # DataQuery object from it.
         if request_data:
             query = get_request_query(request, attrs=request_data.get('query'))
-            
+
             # Now that the DataQuery object is built, read the appropriate
             # attribute from it, returning None if the attribute wasn't found.
             # Since `context` and `view` are the keys used in get_request_view
             # and get_request_context respectively, we can use the key directly
             # to access the context and view properties of the DataQuery model.
             key_object = getattr(query, key, None)
-            
+
             # If the property exists and is not None, then read the json from
             # the object as both DataContext and DataView objects will have a
             # json property. This json will be used as the attributes to
@@ -61,10 +64,7 @@ def _get_request_object(request, attrs=None, klass=None, key=None):
         klass.validate(attrs)
         return klass(json=attrs)
 
-    # Ignore archived objects..
-    kwargs = {
-        'archived': False,
-    }
+    kwargs = {}
 
     # If an authenticated user made the request, filter by the user or
     # fallback to an active session key.
@@ -98,10 +98,11 @@ def _get_request_object(request, attrs=None, klass=None, key=None):
 
 # Partially applied functions for DataView and DataContext. These functions
 # only require the request object and an optional `attrs` dict
-get_request_view = functools.partial(_get_request_object,
-    klass=DataView, key='view')
-get_request_context = functools.partial(_get_request_object,
-    klass=DataContext, key='context')
+get_request_view = functools.partial(
+    _get_request_object, klass=DataView, key='view')
+get_request_context = functools.partial(
+    _get_request_object, klass=DataContext, key='context')
+
 
 def get_request_query(request, attrs=None):
     """
@@ -113,13 +114,13 @@ def get_request_query(request, attrs=None):
             attrs = request.data.get('query')
         elif request.method == 'GET':
             attrs = request.GET.get('query')
-    
+
     # If the `attrs` could not be derived from the request(meaning no query
     # was explicity defined), try to construct the query by deriving a context
     # and view from the request.
     if attrs is None:
         json = {}
-        
+
         context = get_request_context(request)
         if context:
             json['context'] = context.json
@@ -133,7 +134,7 @@ def get_request_query(request, attrs=None):
     # If `attrs` were derived or supplied then validate them and return a
     # DataQuery based off the `attrs`.
     if isinstance(attrs, dict):
-        # We cannot simply validate and create a DataQuery based off the 
+        # We cannot simply validate and create a DataQuery based off the
         # `attrs` as they are now because the context and or view might not
         # contain json but might instead be a pk or some other value. Use the
         # internal helper methods to construct the context and view objects
@@ -150,10 +151,7 @@ def get_request_query(request, attrs=None):
         DataQuery.validate(json)
         return DataQuery(json)
 
-    # Ignore archived objects..
-    kwargs = {
-        'archived': False,
-    }
+    kwargs = {}
 
     # If an authenticated user made the request, filter by the user or
     # fallback to an active session key.
@@ -182,8 +180,9 @@ def get_request_query(request, attrs=None):
     default = DataQuery.objects.get_default_template()
     if default:
         instance.json = default.json
-    return instance  
-     
+    return instance
+
+
 class BaseResource(Resource):
     param_defaults = None
 
@@ -194,7 +193,8 @@ class BaseResource(Resource):
         return super(BaseResource, self).__call__(request, **kwargs)
 
     def process_response(self, request, response):
-        response = super(BaseResource, self).process_response(request, response)
+        response = super(BaseResource, self).process_response(
+            request, response)
         response = cors.patch_response(request, response, self.allowed_methods)
         return response
 
@@ -214,68 +214,78 @@ class BaseResource(Resource):
         "Returns a DataQuery object based on `attrs` or the request."
         return get_request_query(request, attrs=attrs)
 
-class PaginatorParametizer(Parametizer):
-    page = 1
-    per_page = 20
-
-    def clean_page(self, value):
-        return param_cleaners.clean_int(value)
-
-    def clean_per_page(self, value):
-        return param_cleaners.clean_int(value)
+    @property
+    def checks_for_orphans(self):
+        return getattr(settings, 'SERRANO_CHECK_ORPHANED_FIELDS', True)
 
 
-class PaginatorResource(Resource):
-    parametizer = PaginatorParametizer
+class ThrottledResource(BaseResource):
+    def __init__(self, **kwargs):
+        self.rate_limit_count = getattr(
+            settings, 'SERRANO_RATE_LIMIT_COUNT', self.rate_limit_count)
+        self.rate_limit_seconds = getattr(
+            settings, 'SERRANO_RATE_LIMIT_SECONDS', self.rate_limit_seconds)
 
-    def get_paginator(self, queryset, per_page):
-        return Paginator(queryset, per_page=per_page)
+        self.auth_rate_limit_count = getattr(
+            settings, 'SERRANO_AUTH_RATE_LIMIT_COUNT', self.rate_limit_count)
+        self.auth_rate_limit_seconds = getattr(
+            settings, 'SERRANO_AUTH_RATE_LIMIT_SECONDS',
+            self.rate_limit_seconds)
 
-    def get_page_links(self, request, path, page, extra=None):
-        "Returns the page links."
-        uri = request.build_absolute_uri
+        return super(ThrottledResource, self).__init__(**kwargs)
 
-        # format string will be expanded below
-        params = {
-            'page': '{0}',
-            'per_page': '{1}',
-        }
+    def is_too_many_requests(self, request, *arg, **kwargs):
+        limit_count = self.rate_limit_count
+        limit_seconds = self.rate_limit_seconds
 
-        if extra:
-            for key, value in extra.items():
-                # Use the original GET parameter if supplied and if the
-                # cleaned value is valid
-                if key in request.GET and value is not None and value != '':
-                    params.setdefault(key, request.GET.get(key))
+        # Check for an identifier for this request. First, try to use the
+        # user id and then try the session key as a fallback. If this is an
+        # authenticated request then we prepend an indicator to the request
+        # id and use the authenticated limiter settings.
+        if hasattr(request, 'user') and request.user.is_authenticated():
+            request_id = "auth:{0}".format(request.user.id)
+            limit_count = self.auth_rate_limit_count
+            limit_seconds = self.auth_rate_limit_seconds
+        elif request.session.session_key:
+            request_id = request.session.session_key
+        else:
+            # The only time we should reach this point is for
+            # non-authenitcated, cookieless agents(bots). Simply return False
+            # here and let other methods decide how to deal with the bot.
+            return False
 
-        # Stringify parameters. Since these are the original GET params,
-        # they do not need to be encoded
-        pairs = sorted(['{0}={1}'.format(k, v) for k, v in params.items()])
+        # Construct the cache key from the request identifier and lookup
+        # the current cached value for the key. The counts that are stored in
+        # the cache are tuples where the 1st value is the request count for
+        # the given time interval and the 2nd value is the start of the
+        # time interval.
+        cache_key = 'serrano:data_request:{0}'.format(request_id)
+        current_count = cache.get(cache_key)
 
-        # Create path string
-        path_format = '{0}?{1}'.format(path, '&'.join(pairs))
+        # If there is nothing cached for this key then add a new cache value
+        # with a count of 1 and interval starting at the current date and time.
+        # Obviously, if nothing is cached then we can't have had too many
+        # requests as this is the first one so we return False here.
+        if current_count is None:
+            cache.set(cache_key, (1, datetime.now()))
+            return False
+        else:
+            # Calculate the time in seconds between the current date and time
+            # and the interval start from the cached value.
+            interval = (datetime.now() - current_count[1]).seconds
 
-        per_page = page.paginator.per_page
-
-        links = {
-            'self': {
-                'href': uri(path_format.format(page.number, per_page)),
-            },
-            'base': {
-                'href': uri(path),
-            }
-        }
-
-        if page.has_previous():
-            links['prev'] = {
-                'href': uri(path_format.format(page.previous_page_number(), per_page)),
-            }
-
-        if page.has_next():
-            links['next'] = {
-                'href': uri(path_format.format(page.next_page_number(), per_page)),
-            }
-
-        return links
+            # If we have exceeded the interval size then reset the interval
+            # start time and reset the request count to 1 since we are on a
+            # new interval now.
+            if interval > limit_seconds:
+                cache.set(cache_key, (1, datetime.now()))
+                return False
 
+            # Update the request count to account for this request
+            new_count = current_count[0] + 1
+            cache.set(cache_key, (new_count, current_count[1]))
 
+            # Since we are still within the interval, just check if we have
+            # exceeded the request limit or not and return the result of the
+            # comparison.
+            return new_count > limit_count
